#include <iostream>
#include <cdatos.h>
#include <string>
#include <error.h>
#include <matutil.h>

const int maxpoint = 25;
inline double sin3(double x)
{
    return cubo(sin(x));

}
inline double runge(double x)
{
    return 1./(1+cuadrado(x));
}
inline double ff(double x)
{
  return atan(x);
}

CDatos ispace(func f, int n, double x0, double xn)
{
    if (xn < x0)
	error(" x0 debe ser < xn", CERANGO);

    // Crea un conjunto de datos vaco
    CDatos d;
    // igualmente espaciados
    double h = (xn - x0)/(n-1);

    for (int i = 0; i< n; i++){
	double x = x0 + i*h;
	double y = f(x);
	// expande el conjunto de datos
	d.expande(x,y,0);
	x+=h;
    }
    return d;
}

CDatos cheb(func f, int n, double x0, double xn)
{
    if (xn < x0)
	error(" x0 debe ser < xn", CERANGO);


    // Crea un conjunto de datos vaco
    CDatos d;
    // espaciados de acuerdo a los nodos de Chebyshev

    double x = xn;
    n--;
    for (int i = 0; i<= n; i++){
	x = 0.5*(x0 + xn)+ 0.5*(x0 - xn)*cos(PI*(float(i)/n));
	double y = f(x);
	// expande el conjunto de datos
	d.expande(x,y,0);
    }
    return d;
}
    

int main ()
{
    cout << " Cuantos puntos quiere generar? (max "<<maxpoint<<" ) ";
    int npoints;
    cin >> npoints;
    if (npoints > maxpoint) npoints = maxpoint;
    if (npoints < 2) 
	error("Mnimo dos puntos para interpolar", CEFALLO);

    cout << " Qu funcin desea? (1 = sin**3(x), 2 = runge, 3 = F)";
    int ifun; cin >> ifun;
    cout << " Qu espaciado desea? (1 = uniforme, 2 = Chebyshev)";
    int iespa; cin >> iespa;
    cout << " Introduzca intervalo [a,b]" << endl;
    cout << "a? "; double a; cin >> a;
    cout << "b? "; double b; cin >> b;
    string fdatos;
    CDatos d;

    if (ifun == 1){
	if (iespa ==1){
	    d = ispace(sin3, npoints, a, b);
	    fdatos = "sin3.uni";
	}
	else{
	    d = cheb(sin3, npoints, a, b);
	    fdatos = "sin3.che";
	}
    }
    else if (ifun == 2){
	if (iespa ==1){
	    d = ispace(runge, npoints, a, b);
	    fdatos = "runge.uni";
	}
	else{
	    d = cheb(runge, npoints, a, b);
	    fdatos = "runge.che";
	}
    } 
    else if (ifun == 3){
	if (iespa ==1){
	    d = ispace(ff, npoints, a, b);
	    fdatos = "ff.uni";
	}
	else{
	    d = cheb(ff, npoints, a, b);
	    fdatos = "ff.che";
	}
    } 

    cout << "dimension de d = " << d.dimension() <<endl;

    d.escribe(fdatos);
    cout << "conjunto de datos = " << d << endl;


    CDatos cd;
    cd.lee(fdatos);
    cout << "dimension de cd = " << cd.dimension() <<endl;
    cout << "cd = " << cd << endl;

    return 0;
}
