#include <iostream>
#include <iomanip>
#include <interpol.h>
#include <string>
#include <error.h>
#include <matutil.h>

inline double sin3(double x)
{
    return cubo(sin(x));
}
inline double runge(double x)
{
    return 1./(1+cuadrado(x));
}
inline double ff(double x)
{
  return atan(x);
}

int encuentraNodo(const DVector& x,double xp){
    int nodo;
    for (int i = 0; i< x.dimension(); i++){
	if (xp >= x[i] && xp < x[i+1]) nodo = i;
    }
    return nodo;
}
int main ()
{
    // Lee datos a interpolar 
    string fdatos;
    cout << "fichero de datos a interpolar? "; cin >> fdatos; 
    cout << " Qu funcin desea? (1 = sin**3(x), 2 = runge, 3 = F)";
    int ifun; cin >> ifun;

    CDatos cd;
    cd.lee(fdatos);

    // Vectores de abcisas y ordenadas
    DVector x= cd.x();
    DVector y= cd.y();

    // Crea un interpolador
    Interpol xpol;
    // Establece datos
    xpol.eDatos(cd);


    cout  <<"El programa ha leido un conjunto de "
	  <<cd.dimension()<<" nodos" 
	  << endl;

    cout << "nodo inicial x0 = " << x[0] 
	 << " f(x[0])=" << y[0] 
	 << endl;
    cout << "nodo final xf = " << x[cd.dimension()-1] 
	 << " f(x["<<cd.dimension()-1<<"])=" << y[cd.dimension()-1] 
	 << endl << endl;


    // Pide el punto que se desea aproximar.
    cout << "Introduzca el valor del punto que desea aproximar? ";
    double xp; cin >> xp;

    // Encuentra el nodo anterior a xp
    int imin = encuentraNodo(x,xp);
    
    // Interpola polinomios de orden creciente entorno a xp
    for (int imax = imin +1; imax < cd.dimension(); imax++){

	cout << endl << "----------------------------------------"
	     <<"El polinomio se interpola entre los nodos"
	     << " ["<<imin<<"], ["<<imax<<"]"
	     << endl << endl;

	// establece el polinomio
	xpol.ePolinomio(imin,imax);
	int n = imax - imin; // orden del polinomio

	cout << " nmero de nodos a interpolar " << imax - imin + 1 << endl;
	cout << " orden del polinomio interpolador " << n << endl << endl;
	    
	double dd; 
	double pp = xpol.p(xp);
	if (ifun == 1) 
	    dd = abs(pp-sin3(xp))  ;
	else  if (ifun == 2) 
	    dd = abs(pp-runge(xp))  ;
	else  if (ifun == 3) 
	    dd = abs(pp-ff(xp))  ;

	// diferencias entre polinomios de grados sucesivos
	double dif;
	double pold;

	if (n == 1) {
	    dif = -1;
	    pold = pp;
	}
	else{
	    dif = abs(pp - pold);
	    pold = pp;
	}

	cout << "xp = "  
	     << setw(12) << setprecision(10) 
	     << xp 
	     << " p(xp) = "
	     << setw(12) << setprecision(10) 
	     <<pp << " ";
	if (ifun == 1){ 
	    cout << "f(xp) = "  
		 << setw(12) << setprecision(10) 
		 << sin3(xp) << " ";
	}
	else  if (ifun == 2){ 
	    cout << "f(xp) = "  
		 << setw(12) << setprecision(10) 
		 << runge(xp) << " ";
	}
	else  if (ifun == 3){ 
	    cout << "f(xp) = "  
		 << setw(12) << setprecision(10) 
		 << ff(xp) << " ";
	}
	
	cout << endl << "|f(x)-p(x)| = "<<dd<< " "
	     << "|pn(x)-pn-1(x)| = "<<dif << endl;
    }
    
    return 0;
}
